@@ -249,11 +249,11 @@ struct ElementTableScanner {
 };
 
 #ifndef NO_XML_HASH_SCANNER_RETURN
-void *RS_xmlElementTableConverter(void *payload, void *data, xmlChar *name);
-void* RS_xmlEntityTableConverter(void *payload, void *data, xmlChar *name);
+void *RS_xmlElementTableConverter(void *payload, void *data, const xmlChar *name);
+void* RS_xmlEntityTableConverter(void *payload, void *data, const xmlChar *name);
 #else
-void RS_xmlElementTableConverter(void *payload, void *data, xmlChar *name);
-void RS_xmlEntityTableConverter(void *payload, void *data, xmlChar *name);
+void RS_xmlElementTableConverter(void *payload, void *data, const xmlChar *name);
+void RS_xmlEntityTableConverter(void *payload, void *data, const xmlChar *name);
 #endif
 
 
@@ -279,7 +279,8 @@ RS_XML(ProcessElements)(xmlElementTablePtr table, xmlParserCtxtPtr ctxt)
   if(n > 0) {
     USER_OBJECT_ dtdNames = NULL_USER_OBJECT;
 
-    PROTECT(dtdEls = NEW_LIST(n));
+    PROTECT_INDEX ipx;
+    PROTECT_WITH_INDEX(dtdEls = NEW_LIST(n), &ipx);
     PROTECT(dtdNames = NEW_CHARACTER(n));
 #ifdef LIBXML2
  {
@@ -291,6 +292,7 @@ RS_XML(ProcessElements)(xmlElementTablePtr table, xmlParserCtxtPtr ctxt)
    xmlHashScan(table, RS_xmlElementTableConverter, &scanData);
 
    SET_LENGTH(dtdEls, scanData.counter);
+   REPROTECT(dtdEls, ipx);
    SET_LENGTH(dtdNames, scanData.counter);
  }
 #else
@@ -317,7 +319,7 @@ void*
 #else
 void
 #endif
-RS_xmlElementTableConverter(void *payload, void *data, xmlChar *name)
+RS_xmlElementTableConverter(void *payload, void *data, const xmlChar *name)
 {
   struct ElementTableScanner *scanData = (struct ElementTableScanner *)data;
 
@@ -350,7 +352,8 @@ RS_XML(ProcessEntities)(xmlEntitiesTablePtr table, xmlParserCtxtPtr ctxt)
   if(n > 0) {
     USER_OBJECT_ dtdNames;
 
-    PROTECT(dtdEls = NEW_LIST(n));
+    PROTECT_INDEX ipx;
+    PROTECT_WITH_INDEX(dtdEls = NEW_LIST(n), &ipx);
     PROTECT(dtdNames = NEW_CHARACTER(n));
 
 #ifdef LIBXML2
@@ -367,6 +370,7 @@ RS_XML(ProcessEntities)(xmlEntitiesTablePtr table, xmlParserCtxtPtr ctxt)
       */
 
    SET_LENGTH(dtdEls, scanData.counter);
+   REPROTECT(dtdEls, ipx);
    SET_LENGTH(dtdNames, scanData.counter);
 
  }
@@ -391,7 +395,7 @@ void*
 #else
 void
 #endif
-RS_xmlEntityTableConverter(void *payload, void *data, xmlChar *name)
+RS_xmlEntityTableConverter(void *payload, void *data, const xmlChar *name)
 {
   struct ElementTableScanner *scanData = (struct ElementTableScanner *)data;
 
@@ -734,7 +738,7 @@ RS_XML(createDTDAttribute)(xmlAttributePtr val, xmlElementPtr el)
  INTEGER_DATA(VECTOR_ELT(ans, DTD_ATTRIBUTE_DEFAULT_SLOT))[0] =  val->def;
  SET_ENUM_NAME(AttributeDefaultNames, val->def, VECTOR_ELT(ans, DTD_ATTRIBUTE_DEFAULT_SLOT));
 
- if(val->type == XML_ATTRIBUTE_ENUMERATION) {
+ if(val->type == (xmlElementType)XML_ATTRIBUTE_ENUMERATION) {
    SET_VECTOR_ELT(ans, DTD_ATTRIBUTE_DEFAULT_VALUE_SLOT, RS_XML(AttributeEnumerationList)(val->tree, val, el));
  } else {
     SET_VECTOR_ELT(ans, DTD_ATTRIBUTE_DEFAULT_VALUE_SLOT, NEW_CHARACTER(1));
