@@ -72,7 +72,7 @@ typedef struct {
 
 
 static void
-getKeys(void *el, void *data, xmlChar *name)
+getKeys(void *el, void *data, const xmlChar *name)
 {
    HashGatherer *d = (HashGatherer *)data;
    SET_STRING_ELT(d->names, d->pos, COPY_TO_USER_STRING(name));
@@ -119,7 +119,7 @@ R_libxmlTypeTable_lookup(USER_OBJECT_ table, USER_OBJECT_ name, USER_OBJECT_ s_e
    void *p;
 
    t = R_getExternalRef(table, NULL); /* R_libxmlTypeTableGetRef(table); */
-   p = xmlHashLookup(t, CHAR_DEREF(STRING_ELT(name, 0)));
+   p = xmlHashLookup(t, (const xmlChar *)CHAR_DEREF(STRING_ELT(name, 0)));
    ans = R_makeRefObject(p, CHAR_DEREF(STRING_ELT(s_elType, 0)));
 
    return(ans);
@@ -213,7 +213,8 @@ RS_XML_xmlSchemaValidateDoc(SEXP r_schema, SEXP r_doc, SEXP r_options, SEXP r_er
 	R_SchemaValidCallback cbinfo;
 	PROTECT(cbinfo.fun = allocVector(LANGSXP, 2));
 	SETCAR(cbinfo.fun, VECTOR_ELT(r_errorHandlers, 0));
-	xmlSchemaSetValidErrors(ctxt, R_schemaValidityErrorFunc, R_schemaValidityWarningFunc, &cbinfo);
+	xmlSchemaSetValidErrors(ctxt, (xmlSchemaValidityErrorFunc) R_schemaValidityErrorFunc, 
+                                      (xmlSchemaValidityWarningFunc) R_schemaValidityWarningFunc, &cbinfo);
     }
 
     status = xmlSchemaValidateDoc(ctxt, doc);
